@@ -19,7 +19,7 @@ package Astro::ADS::Query;
 #    Alasdair Allan (aa@astro.ex.ac.uk)
 
 #  Revision:
-#     $Id: Query.pm,v 1.17 2001/11/14 00:12:40 aa Exp $
+#     $Id: Query.pm,v 1.24 2011/07/01 bjd Exp $
 
 #  Copyright:
 #     Copyright (C) 2001 University of Exeter. All Rights Reserved.
@@ -67,13 +67,26 @@ use Astro::ADS::Result::Paper;
 use Net::Domain qw(hostname hostdomain);
 use Carp;
 
-'$Revision: 1.17 $ ' =~ /.*:\s(.*)\s\$/ && ($VERSION = $1);
+'$Revision: 1.25 $ ' =~ /.*:\s(.*)\s\$/ && ($VERSION = $1);
+
+# C L A S S   A T T R I B U T E S ------------------------------------------
+{
+	my $_ads_mirror = 'cdsads.u-strasbg.fr';	# this is the default mirror site
+	sub ads_mirror {
+		my ($class, $new_mirror) = @_;
+		$_ads_mirror = $new_mirror if @_ > 1;
+		return $_ads_mirror;
+	}
+}
 
 # C O N S T R U C T O R ----------------------------------------------------
 
 =head1 REVISION
 
-$Id: Query.pm,v 1.17 2001/11/14 00:12:40 aa Exp $
+$Id: Query.pm,v 1.25 2013/08/06 bjd Exp $
+$Id: Query.pm,v 1.24 2009/07/01 bjd Exp $
+$Id: Query.pm,v 1.22 2009/05/01 bjd Exp $
+$Id: Query.pm,v 1.21 2002/09/23 21:07:49 aa Exp $
 
 =head1 METHODS
 
@@ -111,6 +124,7 @@ sub new {
                       BUFFER    => undef }, $class;
 
   # Configure the object
+  # does nothing if no arguments supplied
   $block->configure( @_ );
 
   return $block;
@@ -140,7 +154,7 @@ sub querydb {
   $self->_make_query();
 
   # check for failed connect
-  return undef unless defined $self->{BUFFER};
+  return unless defined $self->{BUFFER};
 
   # return an Astro::ADS::Result object
   return $self->_parse_query();
@@ -164,7 +178,7 @@ sub followup {
   my $self = shift;
 
   # return unless we have arguments
-  return undef unless @_;
+  return unless @_;
 
   my $bibcode = shift;
   my $link_type = shift;
@@ -173,7 +187,7 @@ sub followup {
   $self->_make_followup( $bibcode, $link_type );
 
   # check for failed connect
-  return undef unless defined $self->{BUFFER};
+  return unless defined $self->{BUFFER};
 
   # return an Astro::ADS::Result object
   return $self->_parse_query();
@@ -239,23 +253,30 @@ Return (or set) the current base URL for the ADS query.
 
 if not defined the default URL is cdsads.u-strasbg.fr
 
+As of v1.24, this method sets a class attribute to keep it
+consistant across all objects.  Not terribly thread safe, but
+at least you know where your query is going.
+
 =cut
 
 sub url {
   my $self = shift;
+  my $class = ref($self);	# now re-implemented as a class attribute
 
   # SETTING URL
   if (@_) { 
 
     # set the url option 
     my $base_url = shift; 
-    $self->{URL} = $base_url;
-    $self->{QUERY} = "http://$base_url/cgi-bin/nph-abs_connect?";
-    $self->{FOLLOWUP} = "http://$base_url/cgi-bin/nph-ref_query?";
+    $class->ads_mirror( $base_url );
+    if( defined $base_url ) {
+       $self->{QUERY} = "http://$base_url/cgi-bin/nph-abs_connect?";
+       $self->{FOLLOWUP} = "http://$base_url/cgi-bin/nph-ref_query?";
+    }
   }
 
   # RETURNING URL
-  return $self->{URL};
+  return $class->ads_mirror();
 }
 
 =item B<agent>
@@ -268,7 +289,15 @@ Returns the user agent tag sent by the module to the ADS server.
 
 sub agent {
   my $self = shift;
-  return $self->{USERAGENT}->agent();
+  my $string = shift;
+  if (defined $string) {
+	my $agent = $self->{USERAGENT}->agent();
+	$agent =~ s/(\d+)\s(\[.*\]\s*)?\(/$1 [$string] (/;
+	return $self->{USERAGENT}->agent($agent);
+  }
+  else {
+    return $self->{USERAGENT}->agent();
+  }
 }
 
 # O T H E R   M E T H O D S ------------------------------------------------
@@ -450,6 +479,140 @@ sub bibcode {
   return ${$self->{OPTIONS}}{"bibcode"};
 }
 
+
+=item B<startmonth>
+
+Return (or set) the current starting month of the ADS query.
+
+   $start_month = $query->startmonth();
+   $query->startmonth( "01" );
+
+=cut
+
+sub startmonth {
+  my $self = shift;
+
+  # SETTING STARTING MONTH
+  if (@_) { 
+
+    # set the starting month option  
+    ${$self->{OPTIONS}}{"start_mon"} = shift;
+  }
+
+  # RETURNING STARTING MONTH
+  return ${$self->{OPTIONS}}{"start_mon"};
+
+}
+
+=item B<endmonth>
+
+Return (or set) the current end month of the ADS query.
+
+   $end_month = $query->endmonth();
+   $query->endmonth( "12" );
+
+=cut
+
+sub endmonth {
+  my $self = shift;
+
+  # SETTING END MONTH
+  if (@_) { 
+
+    # set the end month option  
+    ${$self->{OPTIONS}}{"end_mon"} = shift;
+  }
+
+  # RETURNING END MONTH
+  return ${$self->{OPTIONS}}{"end_mon"};
+
+}
+
+=item B<startyear>
+
+Return (or set) the current starting year of the ADS query.
+
+   $start_year = $query->startyear();
+   $query->start_year( "2001" );
+
+=cut
+
+sub startyear {
+  my $self = shift;
+
+  # SETTING START YEAR
+  if (@_) { 
+
+    # set the starting year option  
+    ${$self->{OPTIONS}}{"start_year"} = shift;
+  }
+
+  # RETURNING START YEAR
+  return ${$self->{OPTIONS}}{"start_year"};
+
+}
+
+=item B<endyear>
+
+Return (or set) the current end year of the ADS query.
+
+   $end_year = $query->endyear();
+   $query->end_year( "2002" );
+
+=cut
+
+sub endyear {
+  my $self = shift;
+
+  # SETTING END YEAR
+  if (@_) { 
+
+    # set the end year option  
+    ${$self->{OPTIONS}}{"end_year"} = shift;
+  }
+
+  # RETURNING END YEAR
+  return ${$self->{OPTIONS}}{"end_year"};
+
+}
+
+=item B<journal>
+
+Return (or set) whether refereed, non-refereed (OTHER) or all bibilographic sources (ALL) are returned.
+
+   $query->journal( "REFEREED" );
+   $query->journal( "OTHER" );
+   $query->journal( "ALL" );
+   
+   $journals = $query->journal();
+
+the default is ALL bibilographic sources
+
+=cut
+
+sub journal {
+  my $self = shift;
+
+  # SETTING END YEAR
+  if (@_) { 
+
+    my $source = shift;
+    
+    if ( $source eq "REFEREED" ) {
+       ${$self->{OPTIONS}}{"jou_pick"} = "NO";
+    } elsif ( $source eq "OTHER" ) {
+       ${$self->{OPTIONS}}{"jou_pick"} = "EXCL";
+    } else {
+       ${$self->{OPTIONS}}{"jou_pick"} = "ALL";
+    }  
+
+  }
+
+  # RETURNING END YEAR
+  return ${$self->{OPTIONS}}{"jou_pick"};
+
+}
+
 # C O N F I G U R E -------------------------------------------------------
 
 =back
@@ -470,15 +633,15 @@ Does nothing if the array is not supplied.
 
 sub configure {
   my $self = shift;
+  my $class = ref($self);
 
   # CONFIGURE DEFAULTS
   # ------------------
 
   # define the default base URL
-  $self->{URL} = "cdsads.u-strasbg.fr";
+  my $default_url = $class->ads_mirror();
   
   # define the query URLs
-  my $default_url = $self->{URL};
   $self->{QUERY} = "http://$default_url/cgi-bin/nph-abs_connect?";
   $self->{FOLLOWUP} = "http://$default_url/cgi-bin/nph-ref_query?";
 
@@ -537,19 +700,22 @@ sub configure {
   ${$self->{OPTIONS}}{"bibcode"}          = "";
 
   # Set the data_type option to PORTABLE so our regular expressions work!
+  # Set the return format to LONG so we get full abstracts!
   ${$self->{OPTIONS}}{"data_type"}        = "PORTABLE";
+  ${$self->{OPTIONS}}{"return_fmt"}       = "LONG";
 
   # CONFIGURE FROM ARGUEMENTS
   # -------------------------
 
   # return unless we have arguments
-  return undef unless @_;
+  return unless @_;
 
   # grab the argument list
   my %args = @_;
 
   # Loop over the allowed keys and modify the default query options
   for my $key (qw / Authors AuthorLogic Objects ObjectLogic Bibcode 
+                    StartMonth EndMonth StartYear EndYear Journal
                     Proxy Timeout URL/ ) {
       my $method = lc($key);
       $self->$method( $args{$key} ) if exists $args{$key};
@@ -591,12 +757,16 @@ sub _make_query {
 
    # loop round all the options keys and build the query
    foreach my $key ( keys %{$self->{OPTIONS}} ) {
-      $options = $options . "&$key=${$self->{OPTIONS}}{$key}";
+      # some bibcodes have & and needs to be made "web safe"
+      my $websafe_option = ${$self->{OPTIONS}}{$key};
+      $websafe_option =~ s/&/%26/g;
+      $options = $options . "&$key=$websafe_option";
+
    }
 
    # build final query URL
    $URL = $URL . $options;
-
+   
    # build request
    my $request = new HTTP::Request('GET', $URL);
 
@@ -604,12 +774,51 @@ sub _make_query {
    my $reply = $ua->request($request);
 
    if ( ${$reply}{"_rc"} eq 200 ) {
+   
       # stuff the page contents into the buffer
       $self->{BUFFER} = ${$reply}{"_content"};
+      
+   } elsif ( ${$reply}{"_rc"} eq 500 ) {
+   
+      # we may have a network unreachable, or we may have a no reference
+      # selected error returned by ADS (go figure)
+
+      $self->{BUFFER} = ${$reply}{"_content"};
+      my @buffer = split( /\n/,$self->{BUFFER});
+      chomp @buffer;
+            
+      # assume we have an error unless we can prove otherwise
+      my $error_flag = 1;
+      
+      foreach my $line ( 0 ... $#buffer ) {
+          if( $buffer[$line] =~ "No reference selected" ) {
+       
+             # increment the counter and drop out of the loop
+             $line = $#buffer;
+             $error_flag = 0;
+          }      
+      }
+      
+      # we definately have an error
+      if( $error_flag ) { 
+         $self->{BUFFER} = undef;
+		 my $proxy_string = undef;
+		 if ($proxy_string = $ua->proxy('http')) { substr($proxy_string, 0, 0) = ' using proxy '; }
+		 else { $proxy_string = ' (no proxy)'; }
+         croak("Error ${$reply}{_rc}: Failed to establish network connection to $URL",
+				$proxy_string, "\n");
+      }
+      
    } else {
       $self->{BUFFER} = undef;
-      croak("Error ${$reply}{_rc}: Failed to establish network connection");
+	  my $proxy_string = undef;
+	  if ($proxy_string = $ua->proxy('http')) { substr($proxy_string, 0, 0) = ' using proxy '; }
+	  else { $proxy_string = ' (no proxy)'; }
+      croak("Error ${$reply}{_rc}: Failed to establish network connection to $URL",
+				$proxy_string, "\n");
    }
+   
+   
 }
 
 =item B<_make_followup>
@@ -633,6 +842,7 @@ sub _make_followup {
 
    # which paper?
    my $bibcode = shift;
+   $bibcode =~ s/&/%26/g;	# make ampersands websafe
 
    # which followup?
    my $refs = shift;
@@ -640,10 +850,10 @@ sub _make_followup {
    # which database?
    my $db_key = ${$self->{OPTIONS}}{"db_key"};
    my $data_type = ${$self->{OPTIONS}}{"data_type"};
+   my $fmt = ${$self->{OPTIONS}}{"return_fmt"};
 
    # build the final query URL
-   $URL = $URL .
-          "bibcode=$bibcode&refs=$refs&db_key=$db_key&data_type=$data_type"; 
+   $URL = $URL . "bibcode=$bibcode&refs=$refs&db_key=$db_key&data_type=$data_type&return_fmt=$fmt"; 
 
    # build request
    my $request = new HTTP::Request('GET', $URL);
@@ -656,7 +866,11 @@ sub _make_followup {
       $self->{BUFFER} = ${$reply}{"_content"};
    } else {
       $self->{BUFFER} = undef;
-      croak("Error ${$reply}{_rc}: Failed to establish network connection");
+	  my $proxy_string = undef;
+	  if ($proxy_string = $ua->proxy('http')) { substr($proxy_string, 0, 0) = ' using proxy '; }
+	  else { $proxy_string = ' (no proxy) '; }
+      croak("Error ${$reply}{_rc}: Failed to establish network connection to $URL" .
+			$proxy_string . $self->{BUFFER} ."\n");
    }
 }
 
@@ -682,9 +896,9 @@ sub _parse_query {
   my $paper;
 
   # loop round the returned buffer and stuff the contents into Paper objects
-  my ( $line, $next, $counter );
+  my ( $next, $counter );
   $next = $counter = 0;
-  foreach $line ( 0 ... $#buffer ) {
+  foreach my $line ( 0 ... $#buffer ) {
 
      #     R     Bibcode
      #     T     Title
@@ -705,8 +919,16 @@ sub _parse_query {
 
         # increment the counter and drop out of the loop
         $line = $#buffer;
+        
      }
-
+     
+     # NO ABSTRACT (HTML version)
+     if( $buffer[$line] =~ "No reference selected" ) {
+       
+       # increment the counter and drop out of the loop
+        $line = $#buffer;
+     }
+     
      # NEW PAPER
      if( substr( $buffer[$line], 0, 2 ) eq "%R" ) {
 
@@ -715,6 +937,7 @@ sub _parse_query {
 
         # grab the bibcode
         my $bibcode = substr( $buffer[$counter], 2 );
+        $bibcode =~ s/\s+//g;
 
         # New Astro::ADS::Result::Paper object
         $paper = new Astro::ADS::Result::Paper( Bibcode => $bibcode );
@@ -724,8 +947,9 @@ sub _parse_query {
         # LOOP THROUGH PAPER
         my ( @title, @authors, @affil, @journal, @pubdate, @keywords, 
              @origin, @links, @url, @object, @abstract, @score );
-        while ( substr( $buffer[$counter], 0, 2 ) ne "%R" &&
-                $counter < $#buffer ) {
+        while ( $counter <= $#buffer &&
+                substr( $buffer[$counter], 0, 2 ) ne "%R" ) {
+
 
            # grab the tags
            if( substr( $buffer[$counter], 0, 1 ) eq "%" ) {
@@ -951,10 +1175,10 @@ sub _parse_query {
            }
 
 
+           # set the next paper increment
+           $next = $counter;
            # increment the line counter
            $counter++;
-           # set the next paper increment
-           $next = $counter - 1;
 
         }
 
@@ -1147,6 +1371,19 @@ sub _dump_options {
 
 =end __PRIVATE_METHODS__
 
+=head1 BUGS
+
+=over
+
+=item #35645 filed at rt.cpan.org (Ampersands)
+
+Older versions can't handle ampersands in the bibcode, such as A&A for Astronomy & Astrophysics.
+Fixed for queries in 1.22 - 5/2009.
+Fixed for references in 1.23 - Boyd Duffee E<lt>b dot duffee at isc dot keele dot ac dot ukE<gt>, 7/2011.
+
+=back
+
+
 =head1 COPYRIGHT
 
 Copyright (C) 2001 University of Exeter. All Rights Reserved.
