@@ -19,7 +19,7 @@ package Astro::ADS::Result;
 #    Alasdair Allan (aa@astro.ex.ac.uk)
 
 #  Revision:
-#     $Id: Result.pm,v 1.13 2001/11/10 20:58:43 timj Exp $
+#     $Id: Result.pm,v 1.14 2001/12/03 03:41:45 aa Exp $
 
 #  Copyright:
 #     Copyright (C) 2001 University of Exeter. All Rights Reserved.
@@ -53,13 +53,13 @@ use overload '""' => "stringify";
 
 use Astro::ADS::Result::Paper;
 
-'$Revision: 1.13 $ ' =~ /.*:\s(.*)\s\$/ && ($VERSION = $1);
+'$Revision: 1.14 $ ' =~ /.*:\s(.*)\s\$/ && ($VERSION = $1);
 
 # C O N S T R U C T O R ----------------------------------------------------
 
 =head1 REVISION
 
-$Id: Result.pm,v 1.13 2001/11/10 20:58:43 timj Exp $
+$Id: Result.pm,v 1.14 2001/12/03 03:41:45 aa Exp $
 
 =head1 METHODS
 
@@ -127,7 +127,7 @@ sub pushpaper {
   my $self = shift;
 
   # return unless we have arguments
-  return undef unless @_;
+  return unless @_;
 
   my $paper = shift;
   my $bibcode = $paper->bibcode();
@@ -183,7 +183,7 @@ sub paperbyindex {
   my $self = shift;
 
   # return unless we have arguments
-  return undef unless @_;
+  return unless @_;
 
   my $index = shift;
 
@@ -224,7 +224,7 @@ sub configure {
   my $self = shift;
 
   # return unless we have arguments
-  return undef unless @_;
+  return unless @_;
 
   # grab the argument list
   my %args = @_;
